(* Char -- SML Standard Library *)

type char = char

exception Chr

val minChar : char
val maxChar : char
val maxOrd  : int       

val chr  : int  -> char         (* may raise Chr *)
val ord  : char -> int
val succ : char -> char         (* may raise Chr *)
val pred : char -> char         (* may raise Chr *)

val isLower    : char -> bool   (* contains "abcdefghijklmnopqrstuvwxyz"  *)
val isUpper    : char -> bool   (* contains "ABCDEFGHIJKLMNOPQRSTUVWXYZ"  *)
val isDigit    : char -> bool   (* contains "0123456789"                  *)
val isAlpha    : char -> bool   (* isUpper orelse isLower                 *)
val isHexDigit : char -> bool   (* isDigit orelse contains "abcdefABCDEF" *)
val isAlphaNum : char -> bool   (* isAlpha orelse isDigit                 *)
val isPrint    : char -> bool   (* any printable character (incl. #" ")   *)
val isSpace    : char -> bool   (* contains " \t\r\n\v\f"                 *)
val isPunct    : char -> bool   (* printable, not space or alphanumeric   *) 
val isGraph    : char -> bool   (* (not isSpace) andalso isPrint          *)
val isAscii    : char -> bool   (* ord c < 128                            *)
val isCntrl    : char -> bool   (* control character                      *)

val toLower    : char -> char
val toUpper    : char -> char

val contains    : string -> char -> bool
val notContains : string -> char -> bool

val <  : char * char -> bool
val <= : char * char -> bool
val >  : char * char -> bool
val >= : char * char -> bool
val compare : char * char -> ordering

(* [char] is the type of characters.  

   [minChar] is the least character in the ordering <.

   [maxChar] is the greatest character in the ordering <.

   [maxOrd] is the greatest character code; equals ord(maxChar).

   [chr i] returns the character whose code is i.  Raises Chr if
   i<0 or i>maxOrd.

   [ord c] returns the code of character c.

   [succ c] returns the character immediately following c, or raises
   Chr if c = maxChar.

   [pred c] returns the character immediately preceding c, or raises
   Chr if c = minChar.

   [isLower c] returns true if c is a lowercase letter (a to z).

   [isUpper c] returns true if c is a uppercase letter (A to Z).

   [isDigit c] returns true if c is a decimal digit (0 to 9).

   [isAlpha c] returns true if c is a letter (lowercase or uppercase).

   [isHexDigit c] returns true if c is a hexadecimal digit (0 to 9 or 
   a to f or A to F).

   [isAlphaNum c] returns true if c is alphanumeric (a letter or a
   decimal digit).

   [isPrint c] returns true if c is a printable character (space or visible)

   [isSpace c] returns true if c is a whitespace character (blank, newline,
   tab, vertical tab, new page).

   [isGraph c] returns true if c is a graphical character, that is,
   it is printable and not a whitespace character.

   [isAscii c] returns true if c is a (seven-bit) ASCII character.

   [toLower c] returns the lowercase letter corresponding to c,
   if c is a letter; otherwise returns c.

   [toUpper c] returns the uppercase letter corresponding to c,
   if c is a letter; otherwise returns c.

   [contains s c] returns true if character c occurs in the string s;
   false otherwise.  The function, when applied to s, builds a table
   and returns a function which uses table lookup to decide whether a
   given character is in the string or not.  Hence it is relatively
   expensive to compute  val p = contains s  but very fast to compute 
   p(c) for any given character.

   [notContains s c] returns true if character c does not occur in the
   string s; false otherwise.  Works by construction of a lookup table
   in the same way as the above function.

   [c1 < c2] returns true if ord(c1) < ord(c2), and similarly for <=,
   >, >=.  

   [compare(c1, c2)] returns LESS, EQUAL, or GREATER, according as c1 is
   precedes, equals, or follows c2 in the ordering Char.< .
*)
