(* Word8 -- new basis 1994-11-01, 1995-04-06, 1995-09-07 *)

(* This unit relies on two's complement representation *)

prim_eqtype word;
val wordSize = 8;

(* Invariant for values w of type Word8.word: 0 <= wordToInt w < 256 *)

local
    prim_val orb_       : word -> word -> word      = 2 "or";
    prim_val andb_      : word -> word -> word      = 2 "and";
    prim_val xorb_      : word -> word -> word      = 2 "xor";
    prim_val lshift_    : word -> Word.word -> word = 2 "shift_left";
    prim_val rshiftsig_ : word -> Word.word -> word = 2 "shift_right_signed";
    prim_val rshiftuns_ : word -> Word.word -> word = 2 "shift_right_unsigned";
    prim_val adduns_    : word -> word -> word      = 2 "+intunsig";
    prim_val subuns_    : word -> word -> word      = 2 "-intunsig";
    prim_val muluns_    : word -> word -> word      = 2 "*intunsig";
    prim_val divuns_    : word -> word -> word      = 2 "divunsig";
    prim_val moduns_    : word -> word -> word      = 2 "modunsig";

    prim_val intToWord_ : int -> word = 1 "identity";
    fun norm w = andb_ w (intToWord_ 255);

    prim_val word2int   : Word.word -> int = 1 "identity";
in
    prim_val wordToInt : word -> int = 1 "identity";
    fun signExtend w = if wordToInt w < 128 then (* msbit = 0 *)
			   wordToInt w
		       else		          (* msbit = 1 *)
			   wordToInt (orb_ w (intToWord_ ~256))
    fun intToWord w  = norm (intToWord_ w);

    fun orb (x, y)  = orb_ x y;
    fun andb (x, y) = andb_ x y;
    fun xorb (x, y) = xorb_ x y;
    fun notb x      = norm (xorb_ x (intToWord_ ~1)); 

    fun << (w, k) = 
	if word2int k >= 8 orelse word2int k < 0 then intToWord_ 0
	else norm (lshift_ w k);

    fun >> (w, k) = 
	if word2int k >= 8 orelse word2int k < 0 then intToWord_ 0
	else rshiftuns_ w k;

    fun ~>> (w, k) = 
	if wordToInt w < 128 then	(* msbit = 0: no sign to extend  *)
	    if word2int k >= 8 orelse word2int k < 0 then intToWord_ 0
	    else rshiftuns_ w k
	else				(* msbit = 1: extend, then shift *)
	    if word2int k >= 8 orelse word2int k < 0 then intToWord_ ~1
	    else norm (rshiftsig_ (orb_ w (intToWord_ ~256)) k);

    (* Redefining +, -, *, div, and mod is a horrible idea ... *)

    fun w1  +  w2 = norm (adduns_ w1 w2);
    fun w1  -  w2 = norm (subuns_ w1 w2);
    fun w1  *  w2 = norm (muluns_ w1 w2);
    fun w1 div w2 = divuns_ w1 w2;
    fun w1 mod w2 = moduns_ w1 w2;

    local 
      open StringCvt
      fun skipWSget getc source = getc (skipWS {getc=getc} source)

      (* Below, 48 = Char.ord #"0" and 55 = Char.ord #"A" - 10. *)
      fun decval c = intToWord_ (Char.ord c) - intToWord_ 48;
      fun hexval c = 
	  if #"0" <= c andalso c <= #"9" then 
	      intToWord_ (Char.ord c) - intToWord_ 48
	  else 
	      moduns_ (intToWord_(Char.ord c) - intToWord_ 55) (intToWord_ 32);

      fun prhex i = 
	  if wordToInt i < 10 then Char.chr(wordToInt (i + intToWord_ 48))
	  else Char.chr(wordToInt (i + intToWord_ 55));

      fun conv radix i = 
	  let fun h n res = 
		  if n = intToWord_ 0 then res
		  else h (divuns_ n radix) (prhex (moduns_ n radix) :: res)
	      fun tostr n = h (divuns_ n radix) [prhex (moduns_ n radix)]
	  in String.implode (tostr i) end

    in
      fun scan radix {getc} source =
	  let open StringCvt
	      val (isDigit, factor) = 
		  case radix of
		      BIN => (fn c => (#"0" <= c andalso c <= #"1"),  2)
		    | OCT => (fn c => (#"0" <= c andalso c <= #"7"),  8)
		    | DEC => (Char.isDigit,                          10)
		    | HEX => (Char.isHexDigit,                       16)
	      fun dig1 NONE             = NONE
		| dig1 (SOME (c, rest)) = 
		  let fun digr res src = 
		      case getc src of
			  NONE           => SOME (res, src)
			| SOME (c, rest) => 
			      if isDigit c then 
				  digr(intToWord_ factor*res+hexval c) rest
			      else 
				  SOME (res, src)
		  in 
		      if isDigit c then digr (hexval c) rest 
		      else NONE 
		  end
	  in dig1 (skipWSget getc source) end;

      fun fmt BIN = conv (intToWord_  2)
	| fmt OCT = conv (intToWord_  8)
	| fmt DEC = conv (intToWord_ 10)
	| fmt HEX = conv (intToWord_ 16)
      fun toString w   = conv (intToWord_ 16) w
      fun fromString s = scanString (scan HEX) s
    end (* local for string functions *)

    val op > = fn (w1, w2) => wordToInt w1 > wordToInt w2;
    fun w1 < w2  = w2 > w1;
    fun w1 >= w2 = not (w1 < w2);
    fun w1 <= w2 = not (w1 > w2);
    fun compare (x, y: word) = 
	if x<y then LESS else if x>y then GREATER else EQUAL;
end
